from .new import (
    Translations,
    Keyboard,
    InlineKeyboard,
    CallbackButton,
    UrlButton,
)
from .tg_objects import (
    Update,
    WebhookInfo,
    User,
    Chat,
    ChatMemberT,
    Message,
    MessageId,
    MessageEntity,
    PhotoSize,
    Animation,
    Audio,
    Document,
    Video,
    VideoNote,
    Voice,
    Contact,
    Dice,
    PollOption,
    PollAnswer,
    Poll,
    Location,
    Venue,
    WebAppData,
    ProximityAlertTriggered,
    MessageAutoDeleteTimerChanged,
    VideoChatScheduled,
    VideoChatStarted,
    VideoChatEnded,
    VideoChatParticipantsInvited,
    UserProfilePhotos,
    File,
    WebAppInfo,
    ReplyKeyboardMarkup,
    KeyboardButton,
    KeyboardButtonPollType,
    ReplyKeyboardRemove,
    InlineKeyboardMarkup,
    InlineKeyboardButton,
    LoginUrl,
    CallbackQuery,
    ForceReply,
    ChatPhoto,
    ChatInviteLink,
    ChatAdministratorRights,
    ChatMember,
    ChatMemberOwner,
    ChatMemberAdministrator,
    ChatMemberMember,
    ChatMemberRestricted,
    ChatMemberLeft,
    ChatMemberBanned,
    ChatMemberUpdated,
    ChatJoinRequest,
    ChatPermissions,
    ChatLocation,
    BotCommand,
    BotCommandScope,
    BotCommandScopeDefault,
    BotCommandScopeAllPrivateChats,
    BotCommandScopeAllGroupChats,
    BotCommandScopeAllChatAdministrators,
    BotCommandScopeChat,
    BotCommandScopeChatAdministrators,
    BotCommandScopeChatMember,
    MenuButton,
    MenuButtonCommands,
    MenuButtonWebApp,
    MenuButtonDefault,
    ResponseParameters,
    InputMedia,
    InputMediaPhoto,
    InputMediaVideo,
    InputMediaAnimation,
    InputMediaAudio,
    InputMediaDocument,
    InputFile,
    Sticker,
    StickerSet,
    MaskPosition,
    InlineQuery,
    InlineQueryResult,
    InlineQueryResultArticle,
    InlineQueryResultPhoto,
    InlineQueryResultGif,
    InlineQueryResultMpeg4Gif,
    InlineQueryResultVideo,
    InlineQueryResultAudio,
    InlineQueryResultVoice,
    InlineQueryResultDocument,
    InlineQueryResultLocation,
    InlineQueryResultVenue,
    InlineQueryResultContact,
    InlineQueryResultGame,
    InlineQueryResultCachedPhoto,
    InlineQueryResultCachedGif,
    InlineQueryResultCachedMpeg4Gif,
    InlineQueryResultCachedSticker,
    InlineQueryResultCachedDocument,
    InlineQueryResultCachedVideo,
    InlineQueryResultCachedVoice,
    InlineQueryResultCachedAudio,
    InputMessageContent,
    InputTextMessageContent,
    InputLocationMessageContent,
    InputVenueMessageContent,
    InputContactMessageContent,
    InputInvoiceMessageContent,
    ChosenInlineResult,
    SentWebAppMessage,
    LabeledPrice,
    Invoice,
    ShippingAddress,
    OrderInfo,
    ShippingOption,
    SuccessfulPayment,
    ShippingQuery,
    PreCheckoutQuery,
    PassportData,
    PassportFile,
    EncryptedPassportElement,
    EncryptedCredentials,
    PassportElementError,
    PassportElementErrorDataField,
    PassportElementErrorFrontSide,
    PassportElementErrorReverseSide,
    PassportElementErrorSelfie,
    PassportElementErrorFile,
    PassportElementErrorFiles,
    PassportElementErrorTranslationFile,
    PassportElementErrorTranslationFiles,
    PassportElementErrorUnspecified,
    Game,
    CallbackGame,
    GameHighScore,
)
